function [all_t, time_t] = transpose_heritability(xM,yM,xD,yD, per_t)
%function [all_t, time_t] = transpose_heritability(xM,yM,xD,yD, per_t)
%
% The function computes all the possible heritability indices (HI) over every
% possible transposition between and within MZ- and DZ-twins. The p-value is then
% computed by determining the fraction of obvered HI-value (computed using
% all possible transpostions within groups).
%
% The function follows the tranpsotion method explained in 
% Chung, M.K., Xie, L, Huang, S.-G., Wang, Y., Yan, J., Shen, L. 2019 Rapid 
% acceleration of the permutation test via transpositions, International 
% Workshop on Connectomics in NeuroImaging, Lecture Notes in Computer Science 
% (LNCS) 11848:42-53. http://www.stat.wisc.edu/~mchung/papers/chung.2019.CNI.pdf
%
% INPUT
% xM, yM    : input data of size m x p (m= number of subjects/twins, p=number of data/voxels/edges per subject) 
%             xM(i,:), yM(i,:) is the i-th twin data in MZ-twin.
% xD, xD    : input data of size n x p (n= number of subjects/twins, p=number of data/voxels/edges per subject)
%             xD(i,:), yD(i,:) is the i-th twin data in MZ-twin.
% per_t: number of transpositions
%
% OUTPUT
% all_t:  HI-values of all possible transpositions within and between twins. There should be total
%          per_t number of stat_t values
% time_t:  run time it took to compute the statistics
%
%------------------------------
%Permutation test using tranpositions
%Transpositons within and between twin groups
%
% (C) 2019 Moo K. Chung mkchung@wisc.edu
% University of Wisconsin-Madison
%
% Updated 2020 June 11. Comments, validation

%% VARIABLES
%[xM yM]  MZ-twins
%[xD yD]  DZ-twins
%
%mxM1, myM1, mxD1, myD1 sum functions
%vxM1, vyM1, vxD1, vyD1 variance functions
%covM1, covD1 covaraince functions
%
%These sum and variandce functions are updated for next transpostions
%mxM2, vxM2, covM2 etc... are variables that are computed in the current transpositions.

%% Tested with the following toy example.
%The comments numbers are the ground truth computed sepeartely using the
%MATLAB built-in function corr.m function.

% xM=[2 3 5 1]'
% yM=[6 4 2 5]'
% xD=[3 3 5]'
% yD=[1 2 1]'

tic

m=size(xM,1); %number of MZ-twins
p=size(xM,2); %number of connections
n=size(xD,1); %number of DZ-twins


%% Intital random permutation to speed up computation.
z1=[xM; xD];  %Combine the data
z2=[yM; yD];  %Combine the data
z1=z1(randperm(m+n),:); %random permutation of data zM.
z2=z2(randperm(m+n),:); %random permutation of data zD.

%Unlike the transposition method, you cannot
%save the whole permutations in prior since it takes so much memory
%for large m,n.
xM=z1(1:m,:);xD=z1(m+1:m+n,:); %permuted data is split into group 1 and 2
yM=z2(1:m,:);yD=z2(m+1:m+n,:); %permuted data is split into group 1 and 2


%% Initial condition for sequence of transpositions

%Define sum functions
mxM1=sum(xM);  %11
myM1=sum(yM);  %17
mxD1=sum(xD);  %11
myD1=sum(yD);  %4

%Define variance function omega
omega= inline('sum((x- sum(x)/m).*(y- sum(y)/n))'); %See reference for detail

vxM1=omega(m,n,xM,xM);
vyM1=omega(m,n,yM,yM);
vxD1=omega(m,n,xD,xD);
vyD1=omega(m,n,yD,yD);

%Define covariance function
covM1=omega(m,n,xM,yM);
covD1=omega(m,n,xD,yD);

%Starting twin correlation
CMZ=covM1./sqrt(vxM1.*vyM1); %correlation   -0.8857
CDZ=covD1./sqrt(vxD1.*vyD1); %correlation -0.5

all_t = 2*(CMZ-CDZ); %maximum of heritability index over the whole brain region -0.7714


    
%For HCP data, it is expected we perform more than 50 million transpostions for convergence.
%The number of transpostions should be chosen to gurantee the convergence.


%random indices for transpostion between twins
pi1=randi(m,1,per_t); %random transpostion indices in MZ
pi2=randi(n,1,per_t); %random transpostion indices in DZ
tau=randi(2,1,per_t); %random transpostion for within twin transposition
%tesing
%pi1=[2 4 2]; pi2=[1 2 1]

for k=2:per_t %per
    
    ind1=pi1(k); %index for MZ
    ind2=pi2(k); %index for DZ
    
    %swapping between twin groups. Have to swap vectors [xM yM] and [xD yD]
    %Two possibilities [xM <--> xD, yM <--> yD] or [xM <--> yD, yM <--> xD]
    %This will be randomly done.
    
    %Depending on tau (=1 or 2), we transpose within MZ-twins.
    

    if mod(tau,2)  %transpose within MZ-twin [xM <---> yM] with probability 1/2.
        
        %ind1=pi1(k);  %In testing 4th position changed, ind =4 
        a=xM(ind1,:);  % pick subjects that will do transposition within twin
        b=yM(ind1,:);
        
        xM(ind1,:)=b;  %update XM and yM for next transpostion. 
        yM(ind1,:)=a;
        
        mxM2=mxM1+b-a;  %new mean 15
        myM2=myM1+a-b;
        
        vxM2=vxM1+(mxM1.^2-mxM2.^2)/m + b.^2-a.^2; %new variance  6.75       should match  omega(m,n,xM,xM)=6.75
        vyM2=vyM1+(myM1.^2-myM2.^2)/m + a.^2-b.^2; %new variance 14.75        should match omega(m,n,yM,yM)=14.75   
        covM2=covM1+ ((a-b).^2 - (a-b).*(mxM1-myM1))/m;%new covariance of x and y   should match omega(m,n,xM,yM)=-9.7500

        corrnewM=real(covM2./(sqrt(vxM2.*vyM2))); %new correlation for MZ-twin   should match corr(xM, yM)=-0.9771
        corrnewD=real(covD1./(sqrt(vxD1.*vyD1))); %correlation for DZ is
        %from previuos iteration or the initial condition. We don't do
        %tranpsoition in DZ-twins directly.
        %real.m is used in case sqrt.m gives tiny imaginary numbers.
        
        %the following lines may not needed
        %corrnewM(find(isinf(corrnewM)))=0; %removes possible infinity
        %corrnewM(find(isnan(corrnewM)))=0; %removes possible NaN error
        %corrnewD(find(isinf(corrnewD)))=0; %removes possible infinity
        %corrnewD(find(isnan(corrnewD)))=0; %removes possible NaN error
        
        HI= 2*(corrnewM-corrnewD);
        maxh=[maxh; max(HI,1)]; %max of heritability index
        mxM1=mxM2;myM1=myM2;vxM1=vxM2;vyM1=vyM2;covM1=covM2;%prepare for the next transpositions
        
        %Testing
        %4th position changed from xM=[2 3 5 1]'; yM=[6 4 2 5]'
        %xM=[2 3 5 5]'; yM=[6 4 2 1]'; corr(xM, yM)=-0.9771
  
    end
    
    %% tranpose between MZ- and DZ-twins
    %Data structured like this
    % xM  xD               a   b
    % yM  yD  -- pick -->  c   d
    
    %pick MZ and DZ    
    a=xM(ind1,:);   b=xD(ind2,:);
    c=yM(ind1,:);   d=yD(ind2,:);
      
    %transpose
    xM(ind1,:)=b;  xD(ind2,:)=a;  %xM <--> xD 
    yM(ind1,:)=d;  yD(ind2,:)=c;  %yM <--> yD
    
  % testing 4th MZ and 2nd DZ tranposed
  % Ground truth
  % [a b
  % c d]
  %   1     3
  %   5     2     
%    [xM yM]'
%   corr(xM,yM)
%    [xD yD]'
%    corr(xD, yD)

%    2     3     5     3
%    6     4     2     2
% 
%    -0.7609
% 
%     3     1     5
%     1     5     1
%    -0.8660

         
    mxM2=mxM1+b-a;  %new mean xM <--> xD   should get 13
    myM2=myM1+d-c;                         %should get 14
    
    mxD2=mxD1-b+a ; %new mean yM <--> yD  should get   9
    myD2=myD1-d+c ;                        %should get 11


    vxM2=vxM1+(mxM1.^2-mxM2.^2)/m -a.^2 + b.^2 ; %new variance xM <--> xD     omega(m,m,xM,xM) 4.75
    vyM2=vyM1+(myM1.^2-myM2.^2)/m -c.^2 + d.^2;  %new variance yM <--> yD        omega(m,m,yM,yM) 11 
 
   
    vxD2=vxD1+(mxD1.^2-mxD2.^2)/n -b.^2 + a.^2;    %                        omega(n,n,xD,xD)  8 
    vyD2=vyD1+(myD1.^2-myD2.^2)/n -d.^2 + c.^2;                             %omega(n,n,yD,yD)  10.6667
    
    covM2=covM1+ b.*d - a.*c - (mxM2.*myM2 - mxM1.*myM1)/m ;  %new covariance of xM and yM
    
    covD2=covD1+ a.*c - b.*d - (mxD2.*myD2 - mxD1.*myD1)/n;     %new covariance of xD and yD
    
    corrnewM=real(covM2./(sqrt(vxM2.*vyM2))); %new pearson correlation  should get -0.7609
    corrnewD=real(covD2./(sqrt(vxD2.*vyD2))); %new pearson correlation  should get -0.8660
    
   
    %the following lines may not needed
    %corrnewM(find(isinf(corrnewM)))=0; %removes possible infinity
    %corrnewM(find(isnan(corrnewM)))=0; %removes possible NaN error
    %corrnewD(find(isinf(corrnewD)))=0; %removes possible infinity
    %corrnewD(find(isnan(corrnewD)))=0; %removes possible NaN error
    
  
    HI= 2*(corrnewM-corrnewD);
    
    all_t=[all_t HI]; %heritability index
    
    mxM1=mxM2;myM1=myM2;vxM1=vxM2;vyM1=vyM2;covM1=covM2;%prepare for the next iteration
    mxD1=mxD2;myD1=myD2;vxD1=vxD2;vyD1=vyD2;covD1=covD2;%prepare for the next iteration
end

time_t = toc;











